export default [
    "{ filename: \"DHT/DHT.cpp\", code: \"/* DHT library\\n\\nMIT license\\nwritten by Adafruit Industries\\n*/\\n\\n#include \\\"DHT.h\\\"\\n\\n#define MIN_INTERVAL 2000\\n\\nDHT::DHT(uint8_t pin, uint8_t type, uint8_t count) {\\n  _pin = pin;\\n  _type = type;\\n  #ifdef __AVR\\n    _bit = digitalPinToBitMask(pin);\\n    _port = digitalPinToPort(pin);\\n  #endif\\n  _maxcycles = microsecondsToClockCycles(1000);  // 1 millisecond timeout for\\n                                                 // reading pulses from DHT sensor.\\n  // Note that count is now ignored as the DHT reading algorithm adjusts itself\\n  // basd on the speed of the processor.\\n}\\n\\nvoid DHT::begin(void) {\\n  // set up the pins!\\n  pinMode(_pin, INPUT_PULLUP);\\n  // Using this value makes sure that millis() - lastreadtime will be\\n  // >= MIN_INTERVAL right away. Note that this assignment wraps around,\\n  // but so will the subtraction.\\n  _lastreadtime = -MIN_INTERVAL;\\n  DEBUG_PRINT(\\\"Max clock cycles: \\\"); DEBUG_PRINTLN(_maxcycles, DEC);\\n}\\n\\n//boolean S == Scale.  True == Fahrenheit; False == Celcius\\nfloat DHT::readTemperature(bool S, bool force) {\\n  float f = NAN;\\n\\n  if (read(force)) {\\n    switch (_type) {\\n    case DHT11:\\n      f = data[2];\\n      if(S) {\\n        f = convertCtoF(f);\\n      }\\n      break;\\n    case DHT22:\\n    case DHT21:\\n      f = data[2] & 0x7F;\\n      f *= 256;\\n      f += data[3];\\n      f *= 0.1;\\n      if (data[2] & 0x80) {\\n        f *= -1;\\n      }\\n      if(S) {\\n        f = convertCtoF(f);\\n      }\\n      break;\\n    }\\n  }\\n  return f;\\n}\\n\\nfloat DHT::convertCtoF(float c) {\\n  return c * 1.8 + 32;\\n}\\n\\nfloat DHT::convertFtoC(float f) {\\n  return (f - 32) * 0.55555;\\n}\\n\\nfloat DHT::readHumidity(bool force) {\\n  float f = NAN;\\n  if (read()) {\\n    switch (_type) {\\n    case DHT11:\\n      f = data[0];\\n      break;\\n    case DHT22:\\n    case DHT21:\\n      f = data[0];\\n      f *= 256;\\n      f += data[1];\\n      f *= 0.1;\\n      break;\\n    }\\n  }\\n  return f;\\n}\\n\\n//boolean isFahrenheit: True == Fahrenheit; False == Celcius\\nfloat DHT::computeHeatIndex(float temperature, float percentHumidity, bool isFahrenheit) {\\n  // Using both Rothfusz and Steadman's equations\\n  // http://www.wpc.ncep.noaa.gov/html/heatindex_equation.shtml\\n  float hi;\\n\\n  if (!isFahrenheit)\\n    temperature = convertCtoF(temperature);\\n\\n  hi = 0.5 * (temperature + 61.0 + ((temperature - 68.0) * 1.2) + (percentHumidity * 0.094));\\n\\n  if (hi > 79) {\\n    hi = -42.379 +\\n             2.04901523 * temperature +\\n            10.14333127 * percentHumidity +\\n            -0.22475541 * temperature*percentHumidity +\\n            -0.00683783 * pow(temperature, 2) +\\n            -0.05481717 * pow(percentHumidity, 2) +\\n             0.00122874 * pow(temperature, 2) * percentHumidity +\\n             0.00085282 * temperature*pow(percentHumidity, 2) +\\n            -0.00000199 * pow(temperature, 2) * pow(percentHumidity, 2);\\n\\n    if((percentHumidity < 13) && (temperature >= 80.0) && (temperature <= 112.0))\\n      hi -= ((13.0 - percentHumidity) * 0.25) * sqrt((17.0 - abs(temperature - 95.0)) * 0.05882);\\n\\n    else if((percentHumidity > 85.0) && (temperature >= 80.0) && (temperature <= 87.0))\\n      hi += ((percentHumidity - 85.0) * 0.1) * ((87.0 - temperature) * 0.2);\\n  }\\n\\n  return isFahrenheit ? hi : convertFtoC(hi);\\n}\\n\\nboolean DHT::read(bool force) {\\n  // Check if sensor was read less than two seconds ago and return early\\n  // to use last reading.\\n  uint32_t currenttime = millis();\\n  if (!force && ((currenttime - _lastreadtime) < 2000)) {\\n    return _lastresult; // return last correct measurement\\n  }\\n  _lastreadtime = currenttime;\\n\\n  // Reset 40 bits of received data to zero.\\n  data[0] = data[1] = data[2] = data[3] = data[4] = 0;\\n\\n  // Send start signal.  See DHT datasheet for full signal diagram:\\n  //   http://www.adafruit.com/datasheets/Digital%20humidity%20and%20temperature%20sensor%20AM2302.pdf\\n\\n  // Go into high impedence state to let pull-up raise data line level and\\n  // start the reading process.\\n  digitalWrite(_pin, HIGH);\\n  delay(250);\\n\\n  // First set data line low for 20 milliseconds.\\n  pinMode(_pin, OUTPUT);\\n  digitalWrite(_pin, LOW);\\n  delay(20);\\n\\n  uint32_t cycles[80];\\n  {\\n    // Turn off interrupts temporarily because the next sections are timing critical\\n    // and we don't want any interruptions.\\n    InterruptLock lock;\\n\\n    // End the start signal by setting data line high for 40 microseconds.\\n    digitalWrite(_pin, HIGH);\\n    delayMicroseconds(40);\\n\\n    // Now start reading the data line to get the value from the DHT sensor.\\n    pinMode(_pin, INPUT_PULLUP);\\n    delayMicroseconds(10);  // Delay a bit to let sensor pull data line low.\\n\\n    // First expect a low signal for ~80 microseconds followed by a high signal\\n    // for ~80 microseconds again.\\n    if (expectPulse(LOW) == 0) {\\n      DEBUG_PRINTLN(F(\\\"Timeout waiting for start signal low pulse.\\\"));\\n      _lastresult = false;\\n      return _lastresult;\\n    }\\n    if (expectPulse(HIGH) == 0) {\\n      DEBUG_PRINTLN(F(\\\"Timeout waiting for start signal high pulse.\\\"));\\n      _lastresult = false;\\n      return _lastresult;\\n    }\\n\\n    // Now read the 40 bits sent by the sensor.  Each bit is sent as a 50\\n    // microsecond low pulse followed by a variable length high pulse.  If the\\n    // high pulse is ~28 microseconds then it's a 0 and if it's ~70 microseconds\\n    // then it's a 1.  We measure the cycle count of the initial 50us low pulse\\n    // and use that to compare to the cycle count of the high pulse to determine\\n    // if the bit is a 0 (high state cycle count < low state cycle count), or a\\n    // 1 (high state cycle count > low state cycle count). Note that for speed all\\n    // the pulses are read into a array and then examined in a later step.\\n    for (int i=0; i<80; i+=2) {\\n      cycles[i]   = expectPulse(LOW);\\n      cycles[i+1] = expectPulse(HIGH);\\n    }\\n  } // Timing critical code is now complete.\\n\\n  // Inspect pulses and determine which ones are 0 (high state cycle count < low\\n  // state cycle count), or 1 (high state cycle count > low state cycle count).\\n  for (int i=0; i<40; ++i) {\\n    uint32_t lowCycles  = cycles[2*i];\\n    uint32_t highCycles = cycles[2*i+1];\\n    if ((lowCycles == 0) || (highCycles == 0)) {\\n      DEBUG_PRINTLN(F(\\\"Timeout waiting for pulse.\\\"));\\n      _lastresult = false;\\n      return _lastresult;\\n    }\\n    data[i/8] <<= 1;\\n    // Now compare the low and high cycle times to see if the bit is a 0 or 1.\\n    if (highCycles > lowCycles) {\\n      // High cycles are greater than 50us low cycle count, must be a 1.\\n      data[i/8] |= 1;\\n    }\\n    // Else high cycles are less than (or equal to, a weird case) the 50us low\\n    // cycle count so this must be a zero.  Nothing needs to be changed in the\\n    // stored data.\\n  }\\n\\n  DEBUG_PRINTLN(F(\\\"Received:\\\"));\\n  DEBUG_PRINT(data[0], HEX); DEBUG_PRINT(F(\\\", \\\"));\\n  DEBUG_PRINT(data[1], HEX); DEBUG_PRINT(F(\\\", \\\"));\\n  DEBUG_PRINT(data[2], HEX); DEBUG_PRINT(F(\\\", \\\"));\\n  DEBUG_PRINT(data[3], HEX); DEBUG_PRINT(F(\\\", \\\"));\\n  DEBUG_PRINT(data[4], HEX); DEBUG_PRINT(F(\\\" =? \\\"));\\n  DEBUG_PRINTLN((data[0] + data[1] + data[2] + data[3]) & 0xFF, HEX);\\n\\n  // Check we read 40 bits and that the checksum matches.\\n  if (data[4] == ((data[0] + data[1] + data[2] + data[3]) & 0xFF)) {\\n    _lastresult = true;\\n    return _lastresult;\\n  }\\n  else {\\n    DEBUG_PRINTLN(F(\\\"Checksum failure!\\\"));\\n    _lastresult = false;\\n    return _lastresult;\\n  }\\n}\\n\\n// Expect the signal line to be at the specified level for a period of time and\\n// return a count of loop cycles spent at that level (this cycle count can be\\n// used to compare the relative time of two pulses).  If more than a millisecond\\n// ellapses without the level changing then the call fails with a 0 response.\\n// This is adapted from Arduino's pulseInLong function (which is only available\\n// in the very latest IDE versions):\\n//   https://github.com/arduino/Arduino/blob/master/hardware/arduino/avr/cores/arduino/wiring_pulse.c\\nuint32_t DHT::expectPulse(bool level) {\\n  uint32_t count = 0;\\n  // On AVR platforms use direct GPIO port access as it's much faster and better\\n  // for catching pulses that are 10's of microseconds in length:\\n  #ifdef __AVR\\n    uint8_t portState = level ? _bit : 0;\\n    while ((*portInputRegister(_port) & _bit) == portState) {\\n      if (count++ >= _maxcycles) {\\n        return 0; // Exceeded timeout, fail.\\n      }\\n    }\\n  // Otherwise fall back to using digitalRead (this seems to be necessary on ESP8266\\n  // right now, perhaps bugs in direct port access functions?).\\n  #else\\n    while (digitalRead(_pin) == level) {\\n      if (count++ >= _maxcycles) {\\n        return 0; // Exceeded timeout, fail.\\n      }\\n    }\\n  #endif\\n\\n  return count;\\n}\\n\" }",
    "{ filename: \"DHT/DHT.h\", code: \"/* DHT library\\n\\nMIT license\\nwritten by Adafruit Industries\\n*/\\n#ifndef DHT_H\\n#define DHT_H\\n\\n#if ARDUINO >= 100\\n #include \\\"Arduino.h\\\"\\n#else\\n #include \\\"WProgram.h\\\"\\n#endif\\n\\n\\n// Uncomment to enable printing out nice debug messages.\\n//#define DHT_DEBUG\\n\\n// Define where debug output will be printed.\\n#define DEBUG_PRINTER Serial\\n\\n// Setup debug printing macros.\\n#ifdef DHT_DEBUG\\n  #define DEBUG_PRINT(...) { DEBUG_PRINTER.print(__VA_ARGS__); }\\n  #define DEBUG_PRINTLN(...) { DEBUG_PRINTER.println(__VA_ARGS__); }\\n#else\\n  #define DEBUG_PRINT(...) {}\\n  #define DEBUG_PRINTLN(...) {}\\n#endif\\n\\n// Define types of sensors.\\n#define DHT11 11\\n#define DHT22 22\\n#define DHT21 21\\n#define AM2301 21\\n\\n\\nclass DHT {\\n  public:\\n   DHT(uint8_t pin, uint8_t type, uint8_t count=6);\\n   void begin(void);\\n   float readTemperature(bool S=false, bool force=false);\\n   float convertCtoF(float);\\n   float convertFtoC(float);\\n   float computeHeatIndex(float temperature, float percentHumidity, bool isFahrenheit=true);\\n   float readHumidity(bool force=false);\\n   boolean read(bool force=false);\\n\\n private:\\n  uint8_t data[5];\\n  uint8_t _pin, _type;\\n  #ifdef __AVR\\n    // Use direct GPIO access on an 8-bit AVR so keep track of the port and bitmask\\n    // for the digital pin connected to the DHT.  Other platforms will use digitalRead.\\n    uint8_t _bit, _port;\\n  #endif\\n  uint32_t _lastreadtime, _maxcycles;\\n  bool _lastresult;\\n\\n  uint32_t expectPulse(bool level);\\n\\n};\\n\\nclass InterruptLock {\\n  public:\\n   InterruptLock() {\\n    noInterrupts();\\n   }\\n   ~InterruptLock() {\\n    interrupts();\\n   }\\n\\n};\\n\\n#endif\\n\" }"
];